#include "er.h"
#include "cfg_common.h"
#include "PDBFont.h"

extern node_l *fontCache;

unsigned short int SwapBytes(unsigned short int in)
{
    return ((in-((in>>8)<<8))<<8)+(in>>8);
}

unsigned char MixBytes(unsigned char a, unsigned char b, unsigned char operation)
{
    switch (operation)
    {
    case 0: // just overwrite
        return b;
        break;
    case 1: // darker
        return a&b;
        break;
    case 2: // brighter
        return a|b;
        break;
    }
    return 0;
}

unsigned char CompressColors(unsigned short int in, unsigned char gamma_idx)
{
    unsigned char in0, in1, in2, in3;
    unsigned char out0, out1, out2, out3;
    unsigned char mapping[16];
    switch (gamma_idx)
    {
    case 1: //gamma = 0.25
        mapping[0] = 0;
        mapping[1] = 0;
        mapping[2] = 0;
        mapping[3] = 0;
        mapping[4] = 0;
        mapping[5] = 0;
        mapping[6] = 0;
        mapping[7] = 0;
        mapping[8] = 0;
        mapping[9] = 0;
        mapping[10]= 0;
        mapping[11]= 0;
        mapping[12]= 1;
        mapping[13]= 1;
        mapping[14]= 2;
        mapping[15]= 3;
        break;
    case 2: // gamma = 0.50
        mapping[0] = 0;
        mapping[1] = 0;
        mapping[2] = 0;
        mapping[3] = 0;
        mapping[4] = 0;
        mapping[5] = 0;
        mapping[6] = 0;
        mapping[7] = 0;
        mapping[8] = 0;
        mapping[9] = 1;
        mapping[10]= 1;
        mapping[11]= 1;
        mapping[12]= 1;
        mapping[13]= 2;
        mapping[14]= 2;
        mapping[15]= 3;
        break;
    case 3: // gamma = 0.75
        mapping[0] = 0;
        mapping[1] = 0;
        mapping[2] = 0;
        mapping[3] = 0;
        mapping[4] = 0;
        mapping[5] = 0;
        mapping[6] = 1;
        mapping[7] = 1;
        mapping[8] = 1;
        mapping[9] = 1;
        mapping[10]= 2;
        mapping[11]= 2;
        mapping[12]= 2;
        mapping[13]= 3;
        mapping[14]= 3;
        mapping[15]= 3;
        break;
    case 0: //default value
    case 4: // gamma = 1.00
        mapping[0] = 0;
        mapping[1] = 0;
        mapping[2] = 0;
        mapping[3] = 0;
        mapping[4] = 1;
        mapping[5] = 1;
        mapping[6] = 1;
        mapping[7] = 1;
        mapping[8] = 2;
        mapping[9] = 2;
        mapping[10]= 2;
        mapping[11]= 2;
        mapping[12]= 3;
        mapping[13]= 3;
        mapping[14]= 3;
        mapping[15]= 3;
        break;
    case 5: // gamma = 1.25
        mapping[0] = 0;
        mapping[1] = 0;
        mapping[2] = 0;
        mapping[3] = 1;
        mapping[4] = 1;
        mapping[5] = 1;
        mapping[6] = 1;
        mapping[7] = 2;
        mapping[8] = 2;
        mapping[9] = 2;
        mapping[10]= 2;
        mapping[11]= 2;
        mapping[12]= 3;
        mapping[13]= 3;
        mapping[14]= 3;
        mapping[15]= 3;
        break;
    case 6: // gamma = 1.50
        mapping[0] = 0;
        mapping[1] = 0;
        mapping[2] = 1;
        mapping[3] = 1;
        mapping[4] = 1;
        mapping[5] = 1;
        mapping[6] = 2;
        mapping[7] = 2;
        mapping[8] = 2;
        mapping[9] = 2;
        mapping[10]= 2;
        mapping[11]= 3;
        mapping[12]= 3;
        mapping[13]= 3;
        mapping[14]= 3;
        mapping[15]= 3;
        break;
    case 7: // gamma = 1.75
        mapping[0] = 0;
        mapping[1] = 0;
        mapping[2] = 1;
        mapping[3] = 1;
        mapping[4] = 1;
        mapping[5] = 2;
        mapping[6] = 2;
        mapping[7] = 2;
        mapping[8] = 2;
        mapping[9] = 2;
        mapping[10]= 3;
        mapping[11]= 3;
        mapping[12]= 3;
        mapping[13]= 3;
        mapping[14]= 3;
        mapping[15]= 3;
        break;
    case 8: // gamma = 2.00
        mapping[0] = 0;
        mapping[1] = 1;
        mapping[2] = 1;
        mapping[3] = 1;
        mapping[4] = 2;
        mapping[5] = 2;
        mapping[6] = 2;
        mapping[7] = 2;
        mapping[8] = 2;
        mapping[9] = 3;
        mapping[10]= 3;
        mapping[11]= 3;
        mapping[12]= 3;
        mapping[13]= 3;
        mapping[14]= 3;
        mapping[15]= 3;
        break;
    case 9: // gamma = 2.25
    case 10: // gamma = 2.5
    case 11: // gamma = 2.75
        mapping[0] = 0;
        mapping[1] = 1;
        mapping[2] = 1;
        mapping[3] = 2;
        mapping[4] = 2;
        mapping[5] = 2;
        mapping[6] = 2;
        mapping[7] = 2;
        mapping[8] = 3;
        mapping[9] = 3;
        mapping[10]= 3;
        mapping[11]= 3;
        mapping[12]= 3;
        mapping[13]= 3;
        mapping[14]= 3;
        mapping[15]= 3;
        break;
    case 12: // gamma = 3.00
        mapping[0] = 0;
        mapping[1] = 1;
        mapping[2] = 2;
        mapping[3] = 2;
        mapping[4] = 2;
        mapping[5] = 2;
        mapping[6] = 2;
        mapping[7] = 3;
        mapping[8] = 3;
        mapping[9] = 3;
        mapping[10]= 3;
        mapping[11]= 3;
        mapping[12]= 3;
        mapping[13]= 3;
        mapping[14]= 3;
        mapping[15]= 3;
        break;
    }

    in0 = (in&0xf000)>>12;
    in1 = (in&0x0f00)>>8;
    in2 = (in&0x00f0)>>4;
    in3 = (in&0x000f);

    out0 = mapping[in0]<<6;
    out1 = mapping[in1]<<4;
    out2 = mapping[in2]<<2;
    out3 = mapping[in3];

    return ~(out0+out1+out2+out3);

}

void InitPDB_symbol(struct PDB_Font *font, int i)
{
    int j, k, pos, size, ff;
    unsigned short int tmp;
    unsigned char *ccc;
    
    tmp = font->symbol[i].size/2;
    if(tmp > fontCacheLeft)
    {
        struct PDB_symbol *rare;
        while(tmp > fontCacheLeft)
        {
            rare = list_remlast(&fontCache);
            free(rare->bytes);
            rare->bytes = NULL;
            
            fontCacheLeft += rare->size/2;
        }
    }
    
    fontCacheLeft -= tmp;
    
    if (list_push(&fontCache, &font->symbol[i])==-1) return;

//--------------------    
    ff = font->fontFile;
    SeekFile(ff, font->symbol[i].offset+16, SEEK_SET);

    font->symbol[i].bytes = calloc(font->symbol[i].size/2);
    
    for (j=0;j<font->symbol[i].size/2;j++)
    {
        ReadFile(ff, &tmp, sizeof(short int), &size);
        tmp=SwapBytes(tmp);
        *(font->symbol[i].bytes+j) = CompressColors(tmp, opt.PDB_gamma);
    }
    //clean symbols
    ccc = font->symbol[i].bytes;
    for (j=0; j<font->symbol[i].height; j++)
    {
        pos = 1;
        for (k=0; k<font->symbol[i].rawbyte; k++)
        {
            if (pos>font->symbol[i].width) *ccc|=192;
            pos++;
            if (pos>font->symbol[i].width) *ccc|=48;
            pos++;
            if (pos>font->symbol[i].width) *ccc|=12;
            pos++;
            if (pos>font->symbol[i].width) *ccc|=3;
            pos++;
            ccc++;
        }
    }
}

struct PDB_Font *InitPDBfont(unsigned char *file, unsigned char gamma_idx)
{
    if (LOG) log("<InitPDBFont>\0");
    struct PDB_Font *font;
    int i, ff, size;
    unsigned short int tmp;
    unsigned char c0, c1, c2, c3;
    char cc;
    unsigned char creator[4], tmp_path[256];
    
    memset(&tmp_path,0,256);
    strcpy(tmp_path,"B:\\System\\Fonts\\\0");
    strcat(tmp_path, file);

    ff = OpenFile(tmp_path);
    if (ff==0) return 0;
    
    font = calloc(sizeof(struct PDB_Font));
    memset(font->symbol, 0, sizeof(struct PDB_symbol)*225);

    font->fontFile = ff;
    
    //name
    ReadFile(ff, font->name, 32, &size);
    if (size==0) return 0;

    //creator
    SeekFile(ff, 64, SEEK_SET);
    ReadFile(ff, creator, 4, &size);
    if ((creator[0]!='P')||(creator[1]!='A')||(creator[2]!='F')||(creator[3]!='I')) return 0;

    //offset
    SeekFile(ff, 78, SEEK_SET);
    for (i=0;i<224;i++)
    {
        ReadFile(ff, &c3, 1, &size);
        ReadFile(ff, &c2, 1, &size);
        ReadFile(ff, &c1, 1, &size);
        ReadFile(ff, &c0, 1, &size);
        font->symbol[i].offset=(c3<<24)+(c2<<16)+(c1<<8)+c0;
        ReadFile(ff, &tmp, sizeof(short int), &size);
        ReadFile(ff, &tmp, sizeof(short int), &size);
    }

    //font properties
    SeekFile(ff, font->symbol[0].offset, SEEK_SET);
    ReadFile(ff, &cc, 1, &size);//Version
    ReadFile(ff, &cc, 1, &size);//Size
    font->size = cc;
    ReadFile(ff, &cc, 1, &size);//Ital
    font->ital = cc;
    ReadFile(ff, &cc, 1, &size);//Bold
    font->bold = cc;
    ReadFile(ff, &cc, 1, &size);//Height
    font->height = cc;
    ReadFile(ff, &cc, 1, &size);//Ascent
    font->ascent = cc;
    ReadFile(ff, &cc, 1, &size);//Leading
    font->leading = cc;
    ReadFile(ff, &cc, 1, &size);//Enc.
    font->enc = cc;

    //symbol properties
    for (i=0;i<224;i++)
    {
        ReadFile(ff, &cc, 1, &size);//TopSkip
        font->symbol[i].top_skip = cc;
    }
    for (i=0;i<224;i++)
    {
        ReadFile(ff, &cc, 1, &size);//LeftSkip
        font->symbol[i].left_skip = cc;
    }
    for (i=0;i<224;i++)
    {
        ReadFile(ff, &cc, 1, &size);//Advance
        font->symbol[i].advance = cc;
    }

    //symbol bytes
    for(i=1;i<224;i++)
    {
        SeekFile(ff, font->symbol[i].offset, SEEK_SET);
        ReadFile(ff, &tmp, sizeof(short int), &size);
        tmp=SwapBytes(tmp);
        font->symbol[i].width = tmp;
        ReadFile(ff, &tmp, sizeof(short int), &size);
        tmp=SwapBytes(tmp);
        font->symbol[i].height = tmp;
        ReadFile(ff, &tmp, sizeof(short int), &size);
        tmp=SwapBytes(tmp);
        font->symbol[i].rawbyte = tmp/2;
        font->symbol[i].size = font->symbol[i].height*tmp;	
    }

    if (LOG) log("</InitPDBFont>\0");
    return font;
}

struct PDB_Font *ClosePDBfont(struct PDB_Font *font)
{
    if (LOG) log("<ClosePDBFont>\0");
    if (font!=NULL)
    {
        int i;
        for(i=1;i<224;i++)
        {
            if(font->symbol[i].bytes!=NULL)
            {
                list_popup(&fontCache, &font->symbol[i]);
                list_pop(&fontCache);

                free(font->symbol[i].bytes);
                font->symbol[i].bytes=NULL;

                fontCacheLeft += font->symbol[i].size/2;
            }
        }
        
        CloseFile(font->fontFile);
        
        free(font);
        font=NULL;
    }
    if (LOG) log("</ClosePDBFont>\0");
    return font;
}

void DrawByte(int win, int x, int y, unsigned char byte, unsigned char operation)
{
    unsigned short int a, b;
    unsigned char shift;
    struct window *W;
    W = WindowById(win);

    int byte_x, byte_xx;
    unsigned char *offset;

    byte_x = x/4;
    byte_xx= x - byte_x*4;

    offset = W->buffer + W->scanline*y + byte_x;

    shift = 8-byte_xx*2;
    a = ((*offset)<<8)+(*(offset+1));
    b = byte<<shift;
    // operation should be here
    b|=(~(255<<shift));
    a&=b;
    //
    *offset = a>>8;
    *(offset+1) = a;
}

unsigned short int PrintChar(int win, int x, int y, unsigned char symb, struct PDB_Font *font, unsigned char operation)
{
    int i, j;
    unsigned char *offset;
    unsigned char  cc;

    cc = symb - 32;
    if (cc>0) 
    {    
        if((font->symbol[cc].bytes==NULL)||((int)font->symbol[cc].bytes==-1))
        {
            InitPDB_symbol(font, cc);
        }
        else
        {
            list_popup(&fontCache, &font->symbol[cc]);
        }

        offset = font->symbol[cc].bytes;
        x += font->symbol[cc].left_skip;
        y += font->symbol[cc].top_skip;
        y -= font->ascent;
        
        for (j=0; j<font->symbol[cc].height; j++)
        {
            for (i=0; i<font->symbol[cc].rawbyte; i++)
            {
                DrawByte(win, x+i*4, y+j, *offset, operation);
                offset++;
            }
        }
        return font->symbol[cc].width + font->symbol[cc].left_skip + font->leading;
    }
    if (cc==0) return font->symbol[78].width; //usual way to define the average width of the font-symbol by letter 'n'
    return 0;
}

void Print(int win, int x, int y, unsigned char *text, struct PDB_Font *font)
{
    unsigned char *cc;
    cc = text;
    if (font==NULL)
    {
        y -= curr_font->ascent;
        TextOut(win, x, y, cc);
    }
    else
    {
        while (*cc!=0)
        {
            x += PrintChar(win, x, y, *cc, font, 1);
            cc++;
        }
    }
}

unsigned short int GetCharWidth(unsigned char c, struct PDB_Font *PDBfont)
{
    if (PDBfont==0)
    {
        struct charsize cs;
        if (c==32)
        {
            c=110;
            CharSize(&cs, c);
            return cs.width;
        }
        if (c>32)
        {
            CharSize(&cs, c);
            return cs.width;
        }
    }
    else
    {
        if (c==32)return PDBfont->symbol[78].width + PDBfont->symbol[78].left_skip + PDBfont->leading + 1;
        if (c>32) return PDBfont->symbol[c-32].width + PDBfont->symbol[c-32].left_skip + PDBfont->leading + 1;
    }
    return 0;
}

unsigned short int GetCharHeight(unsigned char c, struct PDB_Font *PDBfont)
{
    if (PDBfont==0)
    {
        struct charsize cs;
        CharSize(&cs, c);
        return cs.height;
    }
    else
    {
        if (c>32)
        {
            return PDBfont->height-PDBfont->symbol[c-32].top_skip;
        }
        else
        {
            //to do: calculation of max GetCharHeight(32..255)   ???
            return PDBfont->height;
        }
    }
    return 0;
}

unsigned short int GetFontHeight(struct PDB_Font *PDBfont)
{
    if (PDBfont==0)
    {
        return curr_font->height;
    }
    else
    {
        return PDBfont->height;
    }
    return 0;
}

short int GetFontAscent(struct PDB_Font *PDBfont)
{
    if (PDBfont==0)
    {
        return 0;
    }
    else
    {
        return PDBfont->ascent;
    }
    return 0;
}

unsigned int GetStringWidth(unsigned char *text, struct PDB_Font *PDBfont)
{
    unsigned char *cc;
    unsigned int width=0;
    cc = text;
    while (*cc!=0)
    {
        width += GetCharWidth(*cc, PDBfont);
        cc++;
    }
    return width;
}
