#include "er.h"
#include "messages.h"
#include "actions.h"

// --- GLOBAL VARIABLES ---
struct options_struct opt, opt_new;
// --- file ---
char *filename, *pos_filename, *bkmrk_filename;
char title[64];
char *fb2_authors, *fb2_title;
int version;
int fd;
long int fd_size, prev_pos;
// --- windows ---
struct out_areas out_area;
unsigned char exit_cfg_Ok, exit_cfg_Cancel, exit_prg;
struct dialog_struct dialog;
// --- text parsing ---
struct pages page[NUM_PAGES];
struct data_struct *data;
struct styles_tree *curr_style;
struct statistics *stat;
char *pagebuffer;
int prev_page, curr_page, next_page;
unsigned char allow_prev_page; // just workaround to fix the problems with prev_page
// --- fonts ---
struct font_struct *curr_font, *fonts[MAX_FONTS];
unsigned char sysFont;
int fontCacheLeft;
node_l *fontCache;
// --- bookmarks ---
unsigned char bkmrk_count, bkmrk_curr, bkmrk_goto;
struct bookmarks *bookmark[64];
// --- misc ---
unsigned char config_changed, config_resetStat, help_page, curr_key;
unsigned char file_type, codepage;
unsigned char save_progress;
struct cfg_font sel_font;
unsigned char lock_key;
// --- local
unsigned char m_long_press, m_postponed;
int logfile;
// --- some functions
void FreeStyleTree();

// --- static variables initialisation ---
void init() { }

void InitArea ()
{
    if (LOG) log("<InitArea>\0");
    struct window *W;
    W = WindowById(EINK);

    out_area.x = opt.page.margin_l;
    out_area.y = opt.page.margin_t+opt.statusbar_t.height;
    out_area.width = W->w - opt.page.margin_l - opt.page.margin_r;
    out_area.height = W->h - opt.page.margin_t - opt.page.margin_b
    - opt.statusbar_t.height - opt.statusbar_b.height;
    out_area.max_width = W->w;
    out_area.max_height = W->h;
    if (LOG) log("</InitArea>\0");
}

void ReDraw()
{
    struct event e;
    e.type = EVT_SHOW;
    e.dest = CurrentScreen();
    SendEvent(&e);
}

void log(unsigned char *text)
{
    int size;
lprintf("%s", text);
    WriteFile(logfile, text, strlen(text), &size);
    WriteFile(logfile, "\n", 1, &size);
}

int m_Init(struct event *e)
{
    if (LOG)
    {
        DeleteFile("B:\\er_log.xml");
        logfile = CreateFile("B:\\er_log.xml");
    }
    if (LOG) log("<m_Init>\0");

    filename = GetFileName();
    fd = OpenFile(filename);
    if (fd == 0)
    {
        Alert("Cannot open book", 1, 1);
        CloseScreen();
        return 0;
    }
    fd_size = GetFileSize(fd);

    GetLongName(filename, title);
    char *p;
    p = strrchr(title, '.');
    if (p != NULL) *p = 0;

    pagebuffer = calloc(BUFF_SIZE*2);
    data = calloc(NUM_PAGES*BUFF_SIZE*sizeof(struct data_struct));
    curr_style = calloc(sizeof(struct styles_tree));
    stat = calloc(sizeof(struct statistics));

    file_type = 0;
    codepage = 0;

    fb2_authors = calloc(sizeof(char)*256);
    fb2_title = calloc(sizeof(char)*256);

    version = VERSION;
    fontCacheLeft = FONT_CASH_SIZE;
    fontCache = NULL;

    MkDir("B:\\System");
    MkDir("B:\\System\\Config");
    LoadOptions();

    if ((opt.page.layout==1)||(opt.page.layout==3)) SetLayout(EINK, 1);
    SetLang(opt.lang);

    InitArea();
    ResetStatistics();

    sel_font.id = 0;
    sel_font.result = 0;
    help_page = 1;

    config_changed = 1;
    config_resetStat = 0;

    save_progress = opt.save_every;
    allow_prev_page = 0;
    m_long_press = 0;
    m_postponed =0;

    pos_filename = calloc(strlen(filename)+1);

    bkmrk_filename = calloc(strlen(filename)+1);

    p = strrchr(filename, '.');
    strncpy(pos_filename, filename, p - filename);
    strncpy(bkmrk_filename, filename, p - filename);
    strcat(pos_filename, ".ezh");
    strcat(bkmrk_filename, ".bkm");

    prev_page = 0;
    curr_page = 1;
    next_page = 2;

    LoadPosition(next_page);
    LoadNextPage();
    TurnNextPage();

    prev_pos = page[curr_page].start;

    LoadBookMarks();

    exit_cfg_Ok = 0;
    exit_cfg_Cancel = 0;
    exit_prg = 0;

    lock_key = 0;

    ResetDialog();

    if (LOG) log("</m_Init>\0");
    return 0;
}

int m_Exit(struct event *e)
{
    if (LOG) log("<m_Exit>\0");

    if (fd)
    {
        SavePosition();
        CloseFile(fd);
        if ((opt.page.layout==1)||(opt.page.layout==3)) SetLayout(EINK, 0);

        FreeStyleTree();
        FreeBookMarks();
        FreeFonts();
        list_delete(&fontCache);

        free(pagebuffer);
        pagebuffer=NULL;

        free(data);
        data=NULL;

        free(pos_filename);
        pos_filename=NULL;

        free(bkmrk_filename);
        bkmrk_filename=NULL;

        free(fb2_authors);
        fb2_authors=NULL;

        free(fb2_title);
        fb2_title=NULL;

        free(stat);
        stat=NULL;

        free(curr_style);
        curr_style=NULL;
    }
    if (LOG) log("</m_Exit>\0");
    if (LOG) CloseFile(logfile);
    return 0;
}

int m_Show(struct event *e)
{
    if (LOG) log("<m_Show>\0");
    if ((opt.page.layout==0)||(opt.page.layout==2)) SetLayout(EINK, 0);
    if ((opt.page.layout==1)||(opt.page.layout==3)) SetLayout(EINK, 1);

    if (exit_prg == 1)
    {
        CloseScreen();
        return 0;
    }

    if (dialog.id == 77)
    {
        page[curr_page].end = dialog.result*(fd_size/100);

        LoadNextPage();
        TurnNextPage();
        ParseNextPage(curr_page);
        PaintPage(curr_page);

        LoadNextPage();
        ResetDialog();
        return 0;
    }

    if (dialog.id == 78)
    {
        page[curr_page].end = dialog.result*(stat->byte.read_byte/stat->byte.read_counter);

        LoadNextPage();
        TurnNextPage();
        ParseNextPage(curr_page);
        PaintPage(curr_page);

        LoadNextPage();
        ResetDialog();
        return 0;
    }

    if (dialog.id == 99)
    {
        if (dialog.result == 1)
        {
            CloseScreen();
            return 0;
        }
        ResetDialog();
    }

    if (bkmrk_goto)
    {
        page[curr_page].end = bookmark[bkmrk_curr]->pos;
        LoadNextPage();
        TurnNextPage();
        LoadNextPage();
        allow_prev_page = 0;
        bkmrk_goto = 0;
    }

    if (config_resetStat)
    {
        ResetStatistics();
        config_resetStat = 0;
    }

    if (config_changed)
    {
        ShowProgressBar1(0);
        ShowProgressBar2(0);
        InitArea();
        ParseNextPage(curr_page);
        LoadNextPage();
        config_changed = 0;
    }

    if (fd) PaintPage(curr_page);

    if (LOG) log("</m_Show>\0");
    return 0;
}

int m_KeyPress(struct event *e)
{
    m_long_press=0;
    m_postponed=0;

    int key = e->par1;
    switch (key)
    {
    case KEY_0: // it will be always enter to configuration. I don't want to change it.
        OpenScreen(0, cfgMain_Handler, "Options", 7000);
        break;
    case KEY_SHELF: // it will be always exit from the progrqamm. I don't want to change it.
        e->type = 0;
        if (opt.confirm_exit)
        {
            dialog.id = 99;
            ShowDialog(0, 0, 0, 0, 0, 1, txt.confirm_exit);
        }
        else
        {
            CloseScreen();
        }
        break;
    }
    return 0;
}

int m_KeyRelease(struct event *e)
{
    int key = e->par1;
    if (m_long_press)
    {
        switch (key)
        {
        case KEY_1:
            Do_Release(opt.key_mapping[lKEY_1]);
            break;
        case KEY_2:
            Do_Release(opt.key_mapping[lKEY_2]);
            break;
        case KEY_3:
            Do_Release(opt.key_mapping[lKEY_3]);
            break;
        case KEY_4:
            Do_Release(opt.key_mapping[lKEY_4]);
            break;
        case KEY_5:
            Do_Release(opt.key_mapping[lKEY_5]);
            break;
        case KEY_6:
            Do_Release(opt.key_mapping[lKEY_6]);
            break;
        case KEY_7:
            Do_Release(opt.key_mapping[lKEY_7]);
            break;
        case KEY_8:
            Do_Release(opt.key_mapping[lKEY_8]);
            break;
        case KEY_9:
            Do_Release(opt.key_mapping[lKEY_9]);
            break;
        case KEY_INFO:
            Do_Release(opt.key_mapping[lKEY_A]);
            break;
        case KEY_CONTENTS:
            Do_Release(opt.key_mapping[lKEY_B]);
            break;
        case KEY_ZOOM:
            Do_Release(opt.key_mapping[lKEY_C]);
            break;
        case KEY_PAGE:
            Do_Release(opt.key_mapping[lKEY_D]);
            break;
        case KEY_ROTATE:
            Do_Release(opt.key_mapping[lKEY_E]);
            break;
        case KEY_PREV:
            Do_Release(opt.key_mapping[lKEY_LEFT]);
            break;
        case KEY_NEXT:
            Do_Release(opt.key_mapping[lKEY_RIGHT]);
            break;
        }
        return 0;
    }

    if (!m_postponed)
    {
        lock_key = key;

        switch (key)
        {
        case KEY_1:
            Do_Press(opt.key_mapping[sKEY_1]);
            break;
        case KEY_2:
            Do_Press(opt.key_mapping[sKEY_2]);
            break;
        case KEY_3:
            Do_Press(opt.key_mapping[sKEY_3]);
            break;
        case KEY_4:
            Do_Press(opt.key_mapping[sKEY_4]);
            break;
        case KEY_5:
            Do_Press(opt.key_mapping[sKEY_5]);
            break;
        case KEY_6:
            Do_Press(opt.key_mapping[sKEY_6]);
            break;
        case KEY_7:
            Do_Press(opt.key_mapping[sKEY_7]);
            break;
        case KEY_8:
            Do_Press(opt.key_mapping[sKEY_8]);
            break;
        case KEY_9:
            Do_Press(opt.key_mapping[sKEY_9]);
            break;
        case KEY_INFO:
            Do_Press(opt.key_mapping[sKEY_A]);
            break;
        case KEY_CONTENTS:
            Do_Press(opt.key_mapping[sKEY_B]);
            break;
        case KEY_ZOOM:
            Do_Press(opt.key_mapping[sKEY_C]);
            break;
        case KEY_PAGE:
            Do_Press(opt.key_mapping[sKEY_D]);
            break;
        case KEY_ROTATE:
            Do_Press(opt.key_mapping[sKEY_E]);
            break;
        case KEY_PREV:
            Do_Press(opt.key_mapping[sKEY_LEFT]);
            break;
        case KEY_NEXT:
            Do_Press(opt.key_mapping[sKEY_RIGHT]);
            break;
        }
        m_postponed = 1;
        SendEvent(e);	
    }
    else
    {
        switch (key)
        {
        case KEY_1:
            Do_Release(opt.key_mapping[sKEY_1]);
            break;
        case KEY_2:
            Do_Release(opt.key_mapping[sKEY_2]);
            break;
        case KEY_3:
            Do_Release(opt.key_mapping[sKEY_3]);
            break;
        case KEY_4:
            Do_Release(opt.key_mapping[sKEY_4]);
            break;
        case KEY_5:
            Do_Release(opt.key_mapping[sKEY_5]);
            break;
        case KEY_6:
            Do_Release(opt.key_mapping[sKEY_6]);
            break;
        case KEY_7:
            Do_Release(opt.key_mapping[sKEY_7]);
            break;
        case KEY_8:
            Do_Release(opt.key_mapping[sKEY_8]);
            break;
        case KEY_9:
            Do_Release(opt.key_mapping[sKEY_9]);
            break;
        case KEY_INFO:
            Do_Release(opt.key_mapping[sKEY_A]);
            break;
        case KEY_CONTENTS:
            Do_Release(opt.key_mapping[sKEY_B]);
            break;
        case KEY_ZOOM:
            Do_Release(opt.key_mapping[sKEY_C]);
            break;
        case KEY_PAGE:
            Do_Release(opt.key_mapping[sKEY_D]);
            break;
        case KEY_ROTATE:
            Do_Release(opt.key_mapping[sKEY_E]);
            break;
        case KEY_PREV:
            Do_Release(opt.key_mapping[sKEY_LEFT]);
            break;
        case KEY_NEXT:
            Do_Release(opt.key_mapping[sKEY_RIGHT]);
            break;
        }
        m_postponed = 0;
    }
    return 0;
}

int m_KeyRepeat(struct event *e)
{
    m_long_press = 1;

    int key = e->par1;
    lock_key = key;
    switch (key)
    {
    case KEY_1:
        Do_Press(opt.key_mapping[lKEY_1]);
        break;
    case KEY_2:
        Do_Press(opt.key_mapping[lKEY_2]);
        break;
    case KEY_3:
        Do_Press(opt.key_mapping[lKEY_3]);
        break;
    case KEY_4:
        Do_Press(opt.key_mapping[lKEY_4]);
        break;
    case KEY_5:
        Do_Press(opt.key_mapping[lKEY_5]);
        break;
    case KEY_6:
        Do_Press(opt.key_mapping[lKEY_6]);
        break;
    case KEY_7:
        Do_Press(opt.key_mapping[lKEY_7]);
        break;
    case KEY_8:
        Do_Press(opt.key_mapping[lKEY_8]);
        break;
    case KEY_9:
        Do_Press(opt.key_mapping[lKEY_9]);
        break;
    case KEY_INFO:
        Do_Press(opt.key_mapping[lKEY_A]);
        break;
    case KEY_CONTENTS:
        Do_Press(opt.key_mapping[lKEY_B]);
        break;
    case KEY_ZOOM:
        Do_Press(opt.key_mapping[lKEY_C]);
        break;
    case KEY_PAGE:
        Do_Press(opt.key_mapping[lKEY_D]);
        break;
    case KEY_ROTATE:
        Do_Press(opt.key_mapping[lKEY_E]);
        break;
    case KEY_PREV:
        Do_Press(opt.key_mapping[lKEY_LEFT]);
        break;
    case KEY_NEXT:
        Do_Press(opt.key_mapping[lKEY_RIGHT]);
        break;
    }
    return 0;
}

static const struct evtproc main_evtproc[] =
{
    { EVT_INIT, m_Init },
    { EVT_SHOW, m_Show },
    { EVT_KEYPRESS, m_KeyPress },
    { EVT_KEYRELEASE, m_KeyRelease },
    { EVT_KEYREPEAT, m_KeyRepeat },
    { EVT_EXIT, m_Exit },
    { 0,    NULL },
};

void MainHandler()
{
    struct event evt;
    GetNextEvent(&evt);
    HandleEvent(main_evtproc, &evt);
    TranslateEvent(&evt);
}

int GetBookInfo(struct bookinfo *bi, char *filename)
{
    char *p;
    memset(bi, 0, sizeof(struct bookinfo));
    GetLongName(filename, bi->title);
    p = strrchr(bi->title, '.');
    if (p != NULL) *p = 0;
    strcpy(bi->author, " ");
    bi->filetype = 1;
    return 0;
}
// --- module header ---
EXPORT struct wolfheader WolfHeader =
{
    init,
    NULL,
    NULL,
    0x480000,
    0x808080,
    "Wolf System File V3.x",
    MainHandler,
    GetBookInfo
};
