#ifndef _ER_H_
#define _ER_H_

#include <wolf.h>
#include "version.h"
#include "PDBFont.h"

// --- line&symbol flags ---
#define IS_VISIBLE     0
#define PARAGRAF_START 1
#define PARAGRAF_END   2
#define BOLD           3
#define ITALIC         4
#define HYPH           5
#define NO 0
#define YES 1
// --- fonts ---
#define MAX_FONTS 32
// --- pages def ---
#define NUM_PAGES 3
#define NUM_STYLES 2
#define BUFF_SIZE 2560
#define MAX_LINES 66
// ---
#define CR 13 // carriage return
#define LF 10 // line feed 
#define SPACE 32 // space
// --- DEBUG
#define LOG 0

// --- structures ---
struct lines {
  int offset, count, space;
  int width;
  unsigned char height, ascent;
  unsigned char flags;
};

struct pages {
  unsigned char num_lines;
  long int start, end;
  struct lines line[MAX_LINES];
};

struct data_struct {
  unsigned char symb;
  int offset;
  unsigned char flags;
  unsigned char style;
};

struct styles_tree {
  unsigned char no;
  struct styles_tree *prev;
  struct styles_tree *next;
};

struct out_areas {
  int x, y;
  int width, height;
  int max_width, max_height;
};

struct opt_paragraph {
  char indent;
  char line_spacing;
  char paragraph_spacing;
  unsigned char alignment;
  unsigned char hyphenation;
};

struct font_struct {
  int no; // =0x333 for PDB Font
  unsigned char height;
  unsigned char ascent;
  unsigned char PDB_file[12]; // =0 if it is not PDB
  struct PDB_Font *pointer; // =0 if it is not PDB
};

struct opt_styles {
  unsigned char font_r, font_b, font_i, font_bi;
  struct opt_paragraph paragraph;
};

struct opt_statusbar {
  unsigned char font;
  unsigned char visible;
  unsigned char progress;
  unsigned char battery;
  unsigned char info;
  unsigned char style;
  unsigned char color;
  unsigned char height;
  unsigned char time;
};

struct opt_profiles {
  struct opt_styles style[NUM_STYLES];
};

struct opt_page {
  unsigned char margin_r;
  unsigned char margin_l;
  unsigned char margin_t;
  unsigned char margin_b;
  unsigned char alignment;
  unsigned char layout;
  unsigned char fullscr;
};

struct options_struct {
  int version;
  unsigned char cfg_page;
  unsigned char profile_no;  
  unsigned char style_no;
  unsigned char lang;
  unsigned char confirm_exit;
  unsigned char hyph_before_after;
  unsigned char save_every;
  unsigned char PDB_gamma;
  unsigned char key_mapping[32];
  struct opt_statusbar statusbar_b, statusbar_t;
  struct opt_page page;
  struct opt_profiles profile[2];
};

struct dialog_struct {
  unsigned char id;
  int val;
  int result;
  int def;
  int min;
  int max;
  int step;
  unsigned char type;
  unsigned char case_no;
  char *text;
};

struct cfg_font {
  unsigned char id;
  char result;
};

struct time_statistics {
  long int turn_time_last;
  long int read_time;
  long int read_time_all;
  int read_counter;
};

struct byte_statistics {
  long int turn_byte_last;
  long int read_byte;
  int read_counter;
};

struct statistics {
  struct time_statistics time;
  struct byte_statistics byte;
};

struct bookmarks {
  long int pos;
  unsigned char note[100];
};

// --- FUCTIONS ---
// --- main window ---
void SetDefOptions(struct options_struct *opt_def);
void ShowProgressBar1(long int curpos);
void ShowProgressBar2(long int curpos);
void ResetStatistics();
void FileTypeDef();
void SavePosition();
void LoadPosition(int p);
void TurnNextPage();
void TurnPrevPage();
void LoadNextPage();
void LoadData(int p, unsigned char direction);
void ParseNextPage(int p);
void PaintPage(int p);
void MainHandler();
void DefOptions();
void ReDraw();
void LoadBookMarks();
void SaveBookMarks();
void AddBookMark();
void DeleteBookMark(int i);
void FreeBookMarks();
void BookMark_Handler();
// --- config
void cfgMain_Handler();
void ShowDialog(int val, int def, int min, int max, int step, unsigned char type, char *text);
void ResetDialog();
void SaveOptions();
void LoadOptions();
void DrawTop(int p);
void DrawCase(int case_num, int num_lines, int btn, char *text);
void DrawBot(char *t1, char *t2, char *t3, char *t4, char *t5);
// --- misc ---
void TextOutCenter(int win, int x, int y, int w, int h, int aw, int ah, char *text);
int TextOutMultiLines(int win, int x, int y, int w, char *text, int end);
void SetByte(unsigned char *dest, int nmbr, int val);
int GetByte(unsigned char *source, int nmbr);
void RotateScreen();
void SetLayout(int win, unsigned char layout);
int WordsComp(unsigned char *first, unsigned char *second);
int DefineTag(unsigned char *cc, unsigned char *tag);
int DefineCode(unsigned char *cc, unsigned char *symb);
void SetVersion();
void SetCurrFont(unsigned char no);
void InitFonts();
void FreeFonts();
void SaveFonts(int optfile);
void LoadFonts(int optfile);
void log(unsigned char *text);
void iprintf(char *text, int number);
void cprintf(char *text, char symbol);
void Lock_Handler();
void DrawHBattary(int win, int x, int y);
void DrawVBattary(int win, int x, int y);
char *GetLongFontName(char *file);

#endif
