#include "er.h"
#include "cfg_common.h"
#include "messages.h"

void InitFonts()
{
    if (LOG) log("<InitFonts>\0");

    int i;
    for (i=0;i<MAX_FONTS;i++)
    {
        fonts[i] = calloc(sizeof(struct font_struct));
    }

    fonts[0]->no = 0x100;
    fonts[0]->height = 16;
    fonts[0]->ascent = 12;

    fonts[1]->no = 0x200;
    fonts[1]->height = 24;
    fonts[1]->ascent = 19;

    fonts[2]->no = 0x201;
    fonts[2]->height = 24;
    fonts[2]->ascent = 19;

    fonts[3]->no = 0x202;
    fonts[3]->height = 24;
    fonts[3]->ascent = 19;

    fonts[4]->no = 0x203;
    fonts[4]->height = 24;
    fonts[4]->ascent = 19;

    fonts[5]->no = 0x300;
    fonts[5]->height = 32;
    fonts[5]->ascent = 27;

    fonts[6]->no = 0x301;
    fonts[6]->height = 32;
    fonts[6]->ascent = 27;

    fonts[7]->no = 0x302;
    fonts[7]->height = 32;
    fonts[7]->ascent = 27;

    fonts[8]->no = 0x303;
    fonts[8]->height = 32;
    fonts[8]->ascent = 27;

    fonts[9]->no= 0x400;
    fonts[9]->height = 32;
    fonts[9]->ascent = 27;

    fonts[10]->no= 0x401;
    fonts[10]->height = 32;
    fonts[10]->ascent = 27;

    fonts[11]->no= 0x402;
    fonts[11]->height = 32;
    fonts[11]->ascent = 27;

    fonts[12]->no= 0x403;
    fonts[12]->height = 32;
    fonts[12]->ascent = 27;

    fonts[13]->no= 0x404;
    fonts[13]->height = 32;
    fonts[13]->ascent = 28;

    fonts[14]->no= 0x405;
    fonts[14]->height = 32;
    fonts[14]->ascent = 26;

    fonts[15]->no= 0x406;
    fonts[15]->height = 32;
    fonts[15]->ascent = 25;

    fonts[16]->no= 0x407;
    fonts[16]->height = 32;
    fonts[16]->ascent = 27;
    
    sysFont = 2;
    
    // PDB-Fonts
    for (i=17;i<MAX_FONTS;i++)
    {
        fonts[i]->no = 0x333;
    };

    if (LOG) log("</InitFonts>\0");
}

void FreeFonts()
{
    if (LOG) log("<FreeFonts>\0");
    int i;

    //close open PDB
    for (i=17;i<MAX_FONTS;i++)
    {
        if (fonts[i]->pointer!=NULL) fonts[i]->pointer=ClosePDBfont(fonts[i]->pointer);
    };

    // free all
    for (i=0;i<MAX_FONTS;i++)
    {
        free(fonts[i]);
    }
    if (LOG) log("</FreeFonts>\0");
}

void SetCurrFont(unsigned char no)
{
    curr_font = fonts[no];

    // Empty PDB
    if ((curr_font->no==0x333)&&(curr_font->PDB_file[0]==0))
    {
        curr_font = fonts[sysFont];
    }

    // WOLF-font
    if (curr_font->no!=0x333)
    {
        SetFont(curr_font->no, 0);
        return;
    }

    // Not yet initialized PDB
    if ((curr_font->no==0x333)&&(curr_font->pointer==NULL)&&(curr_font->PDB_file[0]!=0))
    {
        curr_font->pointer = InitPDBfont(curr_font->PDB_file, opt.PDB_gamma);
        if (curr_font->pointer!=NULL)
        {
            curr_font->height = GetCharHeight(' ', curr_font->pointer);
            curr_font->ascent = GetFontAscent(curr_font->pointer);
        }
        else // this situation should never happen, just to be on the safe side
        {
            curr_font = fonts[sysFont];
            SetCurrFont(curr_font->no);
        }
    }
    return;
}

void SaveFonts(int optfile)
{
    if (LOG) log("<SaveFonts>\0");
    int size;
    char i, font_no=0;

    for (i=17;i<MAX_FONTS;i++)
    {
        if (fonts[(int)i]->PDB_file[0]!=0) font_no++;
    };

    WriteFile(optfile, &font_no, 1, &size);
    if (font_no)
    {
        for (i=17;i<MAX_FONTS;i++)
        {
            if (fonts[(int)i]->PDB_file[0]!=0)
            {
                WriteFile(optfile, &i, 1, &size);
                WriteFile(optfile, &fonts[(int)i]->PDB_file[0], 12, &size);
            }
        };
    }
    if (LOG) log("</SaveFonts>\0");
}

void LoadFonts(int optfile)
{
    if (LOG) log("<LoadFonts>\0");
    int size;
    char i, font_no=0, curr_no;

    InitFonts();
    ReadFile(optfile, &font_no, 1, &size);
    for (i=0;i<font_no;i++)
    {
        ReadFile(optfile, &curr_no, 1, &size);
        ReadFile(optfile, &fonts[(int)curr_no]->PDB_file[0], 12, &size);
    }
    if (LOG) log("</LoadFonts>\0");
}
