/* Simple UTF-8 to CP-1251 recoding routine
* Source file: utf8.c
* Change history:
* 10.09.2007 - Original version by Pyk
* 20.09.2007 - Added support for letter "Io"
* 01.02.2008 - Added function to decode only one symbol (Ice)
*
* Copyright (c) 2007 Pykhonin Victor (aka Pyk)
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "utf8.h"

void DecodeUTF8(void *pBuf, int nBufSize, int *nBytesDecoded, int *nCharsDecoded)
{
    unsigned char *pbReadPtr = (unsigned char *)pBuf;
    unsigned char *pbWritePtr = (unsigned char *)pBuf;

    unsigned int wChr;
    unsigned char btChr;

    int nBytesLeft;

    *nBytesDecoded=*nCharsDecoded=0;

    while (*nBytesDecoded<nBufSize)
    {
        nBytesLeft=nBufSize-*nBytesDecoded;

        if ((pbReadPtr[0] & 0x80)==0) // 1 byte
        {
            *(pbWritePtr++)=*(pbReadPtr++);
            (*nCharsDecoded)++;
            (*nBytesDecoded)++;

            if(pbReadPtr[0]==32 && *nBytesDecoded>*nCharsDecoded)
            {
                memset(pbWritePtr, 1, *nBytesDecoded-*nCharsDecoded);
                pbWritePtr+=*nBytesDecoded-*nCharsDecoded;
                *nCharsDecoded = *nBytesDecoded;
            }
            continue;
        }

        if ((pbReadPtr[0] & 0xC0)==0x80) // not synced - skip byte
        {
            *(pbWritePtr++)=1;//   
            (*nBytesDecoded)++;
            pbReadPtr++;
            continue;
        }

        if ((pbReadPtr[0] & 0xE0)==0xC0) // 2 bytes
        {
            if (nBytesLeft<2) break;
            wChr=(pbReadPtr[0]&0x1f)<<6 | (pbReadPtr[1]&0x3f);
            (*nBytesDecoded)+=2;
            pbReadPtr+=2;
        }
        else if ((pbReadPtr[0] & 0xF0)==0xE0) // 3 bytes
        {
            if (nBytesLeft<3) break;
            wChr=(pbReadPtr[0]&0x0f)<<12 | (pbReadPtr[1]&0x3f)<<6 | (pbReadPtr[2]&0x3f);
            (*nBytesDecoded)+=3;
            pbReadPtr+=3;
        }
        else // unhandled symbols (more than 3 bytes)
        {
            pbReadPtr++;
            (*nBytesDecoded)++;
            continue; // next bytes goes to "not synced - skip byte" above
        }

        if (wChr==0xFEFF) // ignore BOM
        continue;

        // Convert wChr to cp-1251 (often used symbols only)
        btChr='?';
        if (wChr>=0x410 && wChr<=0x44f)
            btChr=wChr-0x350;
        else if (wChr<0x100)
            switch (wChr)
            {
            case 0xAB: // Left double angle quot.
                btChr=0xAB;
                break;
            case 0xBB: // Right double angle quot.
                btChr=0xBB;
                break;
            case 0xA0: // NBSP
                btChr=0xA0;
                break;
            case 0xA7: // Section
                btChr=0xA7;
                break;
            case 0xA9: // (C)
                btChr=0xA9;
                break;
            case 0xAD: // Soft hyphen
                btChr=0xAD;
                break;
            case 0xAE: // (R)
                btChr=0xAE;
                break;
            case 0xB0: // Degree
                btChr=0xB1;
                break;
            }
        else if (wChr>=0x2000)
            switch (wChr)
            {
            case 0x2013: // En Dash
                btChr=0x96;
                break;
            case 0x2014: // Em Dash
                btChr=0x97;
                break;
            case 0x2116: // Numero
                btChr=0xB9;
                break;
            case 0x2026: // Ellipsis
                btChr=0x85;
                break;
            case 0x201C: // Left double quot.
                btChr=0x93;
                break;
            case 0x201D: // Right double quot.
                btChr=0x94;
                break;
            case 0x2022: // Bullet
                btChr=0x95;
                break;
            case 0x2018: // Left single quot.
                btChr=0x91;
                break;
            case 0x2019: // Right single quot.
                btChr=0x92;
                break;
            case 0x201A: // Single low-9 quot.
                btChr=0x93;
                break;
            case 0x201B: // Single high-rev.-9 quot.
                btChr=0x94;
                break;
            case 0x201E: // Double low-9 quot.
                btChr=0x84;
                break;
            case 0x2030: // Per mille
                btChr=0x89;
                break;
            case 0x20AC: // Euro
                btChr=0x88;
                break;
            case 0x2122:
                btChr=0x99;
                break; // TM
            }
        else if (wChr==0x401)//
            btChr=0xA8;
        else if (wChr==0x451)//
            btChr=0xB8;
        else if (wChr==0x404)//
            btChr=0xAA;
        else if (wChr==0x454)//
            btChr=0xBA;
        else if (wChr==0x406)//
            btChr=0xB2;
        else if (wChr==0x456)//
            btChr=0xB3;
        else if (wChr==0x407)//
            btChr=0xAF;
        else if (wChr==0x457)//
            btChr=0xBF;
        *(pbWritePtr++)=btChr;
        (*nCharsDecoded)++;
    }
}

void Decode1UTF8(void *pBuf, int nBufSize, int *nBytesDecoded, int *nCharsDecoded)
{
    unsigned char *pbReadPtr = (unsigned char *)pBuf;
    unsigned char *pbWritePtr = (unsigned char *)pBuf;

    unsigned int wChr;
    unsigned char btChr;

    int nBytesLeft;

    *nBytesDecoded=*nCharsDecoded=0;

    while (*nBytesDecoded<nBufSize)
    {
        nBytesLeft=nBufSize-*nBytesDecoded;

        if ((pbReadPtr[0] & 0x80)==0) // 1 byte
        {
            *(pbWritePtr++)=*(pbReadPtr++);
            (*nCharsDecoded)++;
            (*nBytesDecoded)++;

            if(pbReadPtr[0]==32 && *nBytesDecoded>*nCharsDecoded)
            {
                memset(pbWritePtr, 1, *nBytesDecoded-*nCharsDecoded);
                pbWritePtr+=*nBytesDecoded-*nCharsDecoded;
                *nCharsDecoded = *nBytesDecoded;
            }
////////////////////////////////////////////////////////////////////////
//added by Ice: I need only the first character. If you need more - delete this line            
            break;
////////////////////////////////////////////////////////////////////////
            continue;
        }

        if ((pbReadPtr[0] & 0xC0)==0x80) // not synced - skip byte
        {
            *(pbWritePtr++)=1;//   
            (*nBytesDecoded)++;
            pbReadPtr++;
            continue;
        }

        if ((pbReadPtr[0] & 0xE0)==0xC0) // 2 bytes
        {
            if (nBytesLeft<2) break;
            wChr=(pbReadPtr[0]&0x1f)<<6 | (pbReadPtr[1]&0x3f);
            (*nBytesDecoded)+=2;
            pbReadPtr+=2;
        }
        else if ((pbReadPtr[0] & 0xF0)==0xE0) // 3 bytes
        {
            if (nBytesLeft<3) break;
            wChr=(pbReadPtr[0]&0x0f)<<12 | (pbReadPtr[1]&0x3f)<<6 | (pbReadPtr[2]&0x3f);
            (*nBytesDecoded)+=3;
            pbReadPtr+=3;
        }
        else // unhandled symbols (more than 3 bytes)
        {
            pbReadPtr++;
            (*nBytesDecoded)++;
            continue; // next bytes goes to "not synced - skip byte" above
        }

        if (wChr==0xFEFF) // ignore BOM
        continue;

        // Convert wChr to cp-1251 (often used symbols only)
        btChr='?';
        if (wChr>=0x410 && wChr<=0x44f)
            btChr=wChr-0x350;
        else if (wChr<0x100)
            switch (wChr)
            {
            case 0xAB: // Left double angle quot.
                btChr=0xAB;
                break;
            case 0xBB: // Right double angle quot.
                btChr=0xBB;
                break;
            case 0xA0: // NBSP
                btChr=0xA0;
                break;
            case 0xA7: // Section
                btChr=0xA7;
                break;
            case 0xA9: // (C)
                btChr=0xA9;
                break;
            case 0xAD: // Soft hyphen
                btChr=0xAD;
                break;
            case 0xAE: // (R)
                btChr=0xAE;
                break;
            case 0xB0: // Degree
                btChr=0xB1;
                break;
            }
        else if (wChr>=0x2000)
            switch (wChr)
            {
            case 0x2013: // En Dash
                btChr=0x96;
                break;
            case 0x2014: // Em Dash
                btChr=0x97;
                break;
            case 0x2116: // Numero
                btChr=0xB9;
                break;
            case 0x2026: // Ellipsis
                btChr=0x85;
                break;
            case 0x201C: // Left double quot.
                btChr=0x93;
                break;
            case 0x201D: // Right double quot.
                btChr=0x94;
                break;
            case 0x2022: // Bullet
                btChr=0x95;
                break;
            case 0x2018: // Left single quot.
                btChr=0x91;
                break;
            case 0x2019: // Right single quot.
                btChr=0x92;
                break;
            case 0x201A: // Single low-9 quot.
                btChr=0x93;
                break;
            case 0x201B: // Single high-rev.-9 quot.
                btChr=0x94;
                break;
            case 0x201E: // Double low-9 quot.
                btChr=0x84;
                break;
            case 0x2030: // Per mille
                btChr=0x89;
                break;
            case 0x20AC: // Euro
                btChr=0x88;
                break;
            case 0x2122:
                btChr=0x99;
                break; // TM
            }
        else if (wChr==0x401)//
            btChr=0xA8;
        else if (wChr==0x451)//
            btChr=0xB8;
        else if (wChr==0x404)//
            btChr=0xAA;
        else if (wChr==0x454)//
            btChr=0xBA;
        else if (wChr==0x406)//
            btChr=0xB2;
        else if (wChr==0x456)//
            btChr=0xB3;
        else if (wChr==0x407)//
            btChr=0xAF;
        else if (wChr==0x457)//
            btChr=0xBF;
        *(pbWritePtr++)=btChr;
        (*nCharsDecoded)++;
////////////////////////////////////////////////////////////////////////
//added by Ice: I need only the first character. If you need more - delete this line            
            break;
////////////////////////////////////////////////////////////////////////
    }
}
